const configPath = ".\\plugins\\WaJue\\WJPZ\\config.json";
const defaultConfig = {
    "llmoney": true,
    "score": "money",
    "DigRewards": {
        "10": 5
    },
    "ilandDisable": true,
    "rewardSound": "random.totem",
    "cooldownEnabled": true,
    "cooldownDuration": 20000,
    "blockRewards": {},
    "noRewardBlocks": []
};
const config = data.openConfig(configPath, "json", JSON.stringify(defaultConfig));

const digData = {};
const isLand = lxl.import('ILAPI_PosGetLand');
const isLandTrust = lxl.import('ILAPI_IsPlayerTrusted');
const isLandOwner = lxl.import('ILAPI_IsLandOwner');
const isLandPerm = lxl.import('ILAPI_CheckPerm');
let ilandDisable = config.get("ilandDisable");
let noRewardBlocks = new Set(config.get("noRewardBlocks"));
let blockRewards = config.get("blockRewards");
let cooldownDuration = config.get("cooldownDuration");
let cooldownEnabled = config.get("cooldownEnabled");
let rewardSound = config.get("rewardSound");
let llmoney = config.get("llmoney");
let score = config.get("score");
let digRewards = config.get("DigRewards");

mc.listen("onServerStarted", () => {
    log("挖掘奖励插件已加载！");
    registerCommands();
});

mc.listen("onDestroyBlock", (player, block) => {
    const playerName = player.realName;
    const currentTime = Date.now();
    const digInfo = digData[playerName] || initializeDigInfo(playerName);

    if (ilandDisable && !isLandAllowed(block, player)) {
        sendTitle(player, "您没有此领地的权限，无法获得挖掘奖励。");
        return;
    }

    if (noRewardBlocks.has(block.type)) {
        sendTitle(player, "该方块没有挖掘奖励。");
        return;
    }

    if (blockRewards[block.type]) {
        grantReward(player, blockRewards[block.type]);
        return;
    }

    if (cooldownEnabled) {
        if (currentTime - digInfo.lastRewardTime >= cooldownDuration) {
            digInfo.digCount++;
            digInfo.hasShownCooldown = false;

            for (const [key, value] of Object.entries(digRewards)) {
                if (digInfo.digCount >= parseInt(key)) {
                    grantReward(player, value);
                    resetDigInfo(digInfo, currentTime);
                    return;
                }
            }
        } else {
            if (!digInfo.hasShownCooldown) {
                digInfo.hasShownCooldown = true;
                const remainingTime = Math.ceil((digInfo.lastRewardTime + cooldownDuration - currentTime) / 1000);
                sendTitle(player, `您还需要等待${remainingTime}秒才能再次获得奖励。`);
            }
        }
    } else {
        digInfo.digCount++;

        for (const [key, value] of Object.entries(digRewards)) {
            if (digInfo.digCount >= parseInt(key)) {
                grantReward(player, value);
                resetDigInfo(digInfo, currentTime);
                return;
            }
        }
    }
});

function initializeDigInfo(playerName) {
    return digData[playerName] = {
        digCount: 0,
        pendingGold: 0,
        lastRewardTime: 0,
        hasShownCooldown: false
    };
}

function resetDigInfo(digInfo, currentTime) {
    digInfo.digCount = 0;
    digInfo.lastRewardTime = currentTime;
    digInfo.hasShownCooldown = false;
}

function isLandAllowed(block, player) {
    if (ilandDisable) {
        const land = isLand(toRawPos(block.pos));
        if (land !== -1) {
            const xuid = String(player.xuid);
            return isLandTrust(land, xuid) || isLandOwner(land, xuid) || isLandPerm(land, "allow_destroy");
        }
    }
    return true;
}

function toRawPos(pos) {
    return { x: pos.x, y: pos.y, z: pos.z, dimid: pos.dimid };
}

function grantReward(player, rewardAmount) {
    if (llmoney) {
        try {
            money.add(player.xuid, rewardAmount);
            sendTitle(player, `您已获得${rewardAmount}金币！`);
        } catch (e) {
            sendTitle(player, `奖励发放失败：${e.message}`);
        }
    } else {
        player.addScore(score, rewardAmount);
        sendTitle(player, `您已获得${rewardAmount}金币！`);
    }

    if (rewardSound) {
        mc.runcmdEx(`playsound ${rewardSound} ${player.name}`);
    }
}

function sendTitle(player, message) {
    mc.runcmdEx(`title ${player.name} actionbar ${message}`);
}

function showAdminForm(player) {
    if (!player.isOP()) {
        player.tell("只有管理员可以访问这个设置表单。");
        return;
    }

    const form = mc.newCustomForm();
    form.setTitle("§a挖掘奖励设置");
    form.addSwitch("§a使用LLMoney", llmoney);
    form.addInput("§a记分板项", "money", score);
    form.addInput("§a冷却时间 (1000=1秒)", "20000", cooldownDuration.toString());
    form.addSwitch("§a启用冷却", cooldownEnabled);
    form.addInput("§a奖励音效", "random.totem", rewardSound);
    form.addSwitch("§a领地检测", ilandDisable);

   const digRewardKeys = Object.keys(digRewards).join(',');
    const digRewardValues = Object.values(digRewards).join(',');
    form.addInput("§e挖掘奖励次数", "30", digRewardKeys);
    form.addInput("§e挖掘奖励金额", "15", digRewardValues);

    const blockRewardsKeys = Object.keys(blockRewards).join(',');
    const blockRewardsValues = Object.values(blockRewards).join(',');
    form.addInput("§b特殊奖励方块(用英文逗号分隔和下面的奖励金都不需要空格)", "minecraft:diamond_ore", blockRewardsKeys);
    form.addInput("§b特殊奖励金额(价格英文逗号分隔,对应着上面的)", "10", blockRewardsValues);
    form.addInput("§c没有奖励的方块 (ID，用逗号分隔)", "minecraft:stone", [...noRewardBlocks].join(','));

    player.sendForm(form, (player, result) => {
        if (result === null) return;

        llmoney = result[0];
        score = result[1];
        cooldownDuration = parseInt(result[2]);
        cooldownEnabled = result[3];
        rewardSound = result[4];
        ilandDisable = result[5];

        try {
            const keys = result[6].split(',').map(key => key.trim());
            const values = result[7].split(',').map(value => parseInt(value.trim()));
            digRewards = keys.reduce((acc, key, index) => {
                acc[key] = values[index];
                return acc;
            }, {});

            const blockKeys = result[8].split(',').map(key => key.trim());
            const blockValues = result[9].split(',').map(value => parseInt(value.trim()));
            blockRewards = blockKeys.reduce((acc, key, index) => {
                acc[key] = blockValues[index];
                return acc;
            }, {});

            noRewardBlocks = new Set(result[10].split(',').map(item => item.trim()));
        } catch (e) {
            player.tell("挖掘奖励格式错误，请使用正确的格式。");
        }

        config.set("llmoney", llmoney);
        config.set("score", score);
        config.set("cooldownDuration", cooldownDuration);
        config.set("cooldownEnabled", cooldownEnabled);
        config.set("rewardSound", rewardSound);
        config.set("ilandDisable", ilandDisable);
        config.set("DigRewards", digRewards);
        config.set("blockRewards", blockRewards);
        config.set("noRewardBlocks", [...noRewardBlocks]);

        player.tell("配置已更新，设置已即时生效。");
    });
}

function registerCommands() {
    mc.regPlayerCmd("dr", "打开挖掘奖励设置表单", (player, args) => {
        if (player.isOP()) {
            showAdminForm(player);
        } else {
            player.tell("只有管理员可以执行此命令。");
        }
    }, 1);
}

function load() {
    log("======================");
    log("===挖掘奖励加载成功===");
    log("=========作者========");
    log("========ZIXIE=========");
}

load();
